:- module(latin).

:- export
        latin/2,
        none_problem/2,
        problem/2,
        problem_symmetries/2,
        gap_sbds_problem/2,
        gap_sbdd_problem/2,
        isbds_problem/2,
        static_problem/2,
        sbdd_ssb_problem/4.

:- lib(ic).
:- lib(ic_sbds).
:- lib(ic_global).
:- lib(ic_gap_sbds).
:- lib(ic_gap_sbdd).

:- lib(ldsb).
:- lib(ldsb_syms).
:- lib(ldsb_sbds).
:- lib(ldsb_misc).
%:- lib(myutil).
:- lib(ldsb_static).

:- import alldifferent/1 from ic_global.

latin(N, Board) :-
        dim(Board, [N,N]),
        Board[1..N, 1..N] :: 1..N,

        ( for(I, 1, N), param(N, Board) do

          alldifferent(Board[I, 1..N]),
          alldifferent(Board[1..N, I])
        ).

problem_symmetries(Matrix, Syms) :-
        % rows_interchange(Matrix, RowSym),
        % columns_interchange(Matrix, ColSym),
        % values_interchange(Matrix, ValSym),
        % diagonal_reflect(Matrix, DimSym),
        % Syms = [RowSym, ColSym, ValSym, DimSym].
        simplify_symmetries(Matrix,
                            [rows_interchange, columns_interchange, values_interchange, diagonal_reflection],
                            Syms).

none_problem(N, Matrix) :- latin(N, Matrix).

problem(N, Matrix) :-
        latin(N, Matrix),
        problem_symmetries(Matrix, Symmetries),
        ldsb_initialise(Matrix, Symmetries).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

swap_dim([N,N], [I,J], [J,I]).

gap_sbds_problem(N, M) :-
        latin(N,M),
        ic_gap_sbds:sbds_initialise(M, [dim1, dim2], value:1..N,
                        [symmetry(s_n, [dim1], []),
                         symmetry(function(swap_dim), [dim1, dim2], []),
                         symmetry(function(swap_dim), [dim2, value], [])],
                        []).

gap_sbdd_problem(N, M) :-
        latin(N,M),
        ic_gap_sbdd:sbdd_initialise(M, [dim1, dim2], value:1..N,
                        [symmetry(s_n, [dim1], []),
                         symmetry(function(swap_dim), [dim1, dim2], []),
                         symmetry(function(swap_dim), [dim2, value], [])],
                        []).

isbds_problem(N, M) :-
        latin(N,M),
        problem_symmetries(M, Syms),
        convert_symmetries(pairs, M, Syms, SBDSSyms),
        ic_sbds:sbds_initialise(M, 2, SBDSSyms, #=, []).

static_problem(N, Matrix) :-
        latin(N, Matrix),
        problem_symmetries(Matrix, Symmetries),
        post_symmetries(Matrix, Symmetries).

sbdd_ssb_problem(N, Vector, VarPart, ValPart) :-
        latin(N,M),
        problem_symmetries(M, Syms),
        symmetries_to_ssb(M, Syms, Vector, VarPart, ValPart).
