:- module(queens).

:- export
        queens/2,
        none_problem/2,
        problem/2,
        problem_symmetries/2,
        gap_sbds_problem/2,
        gap_sbdd_problem/2,
        isbds_problem/2,
        static_problem/2,
        sbdd_ssb_problem/4.

:- lib(ic).
:- lib(ic_gap_sbds).
:- lib(ic_gap_sbdd).
:- lib(ic_sbds).

:- lib(ldsb).
:- lib(ldsb_syms).
:- lib(ldsb_sbds).
:- lib(ldsb_static).
:- lib(ldsb_misc).

queens(N, Matrix) :-
        length(Board, N),
        Board #:: 1..N,
        ( fromto(Board, [Q1|Cols], Cols, []) do
          ( foreach(Q2, Cols), param(Q1), count(Dist,1,_) do
            noattack(Q1, Q2, Dist)
          )
        ),
        Row =.. [[]|Board],
        % Matrix is a 1xN matrix.
        Matrix = [](Row).

noattack(Q1,Q2,Dist) :-
        Q2 #\= Q1,
        Q2 #\= Dist + Q1,
        Q1 #\= Dist + Q2.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

problem_symmetries(Matrix, Syms) :-
        columns_reflect(Matrix, VarSym),
        values_reflect(Matrix, ValSym),
        Syms = [VarSym, ValSym].

problem(N, Matrix) :-
        queens(N, Matrix),
        problem_symmetries(Matrix, Syms),
        ldsb_initialise(Matrix, Syms).

none_problem(N, Matrix) :- queens(N, Matrix).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gap_sbds_problem(N,M) :-
        queens(N, M),
        M = [](Vector),
        ic_gap_sbds:sbds_initialise(Vector, [q], value:1..N,
                        [symmetry(d_4, [q,value], [])],
                        []).

gap_sbdd_problem(N,M) :-
        queens(N, M),
        M = [](Vector),
        ic_gap_sbdd:sbdd_initialise(Vector, [q], value:1..N,
                        [symmetry(d_4, [q,value], [])],
                        []).

isbds_problem(N, M) :-
        queens(N,M),
        problem_symmetries(M, Syms),
        convert_symmetries(pairs, M, Syms, SBDSSyms),
        ic_sbds:sbds_initialise(M, 2, SBDSSyms, #=, []).

static_problem(N, Matrix) :-
        queens(N, Matrix),
        problem_symmetries(Matrix, Syms),
        post_symmetries(Matrix, Syms).

sbdd_ssb_problem(N, Vector, VarPart, ValPart) :-
        queens(N, Matrix),
        problem_symmetries(Matrix, Syms),
        symmetries_to_ssb(Matrix, Syms, Vector, VarPart, ValPart).
