:- module(queens_bool).

:- export
        queens_bool/2,
        none_problem/2,
        problem/2,
        problem_symmetries/2,
        gap_sbds_problem/2,
        gap_sbdd_problem/2,
        isbds_problem/2,
        static_problem/2,
        sbdd_ssb_problem/4.

:- lib(ic).
:- lib(ic_gap_sbds).
:- lib(ic_gap_sbdd).

:- lib(ldsb).
:- lib(ldsb_syms).
:- lib(ldsb_sbds).
:- lib(ldsb_static).
:- lib(ldsb_misc).

queens_bool(N, A) :-
          dim(A, [N,N]),
          A #:: 0..1,

          (
              for(I,1,N),
              param(A,N) do

              % First two zinc constraints:
              % ( sum (j in rg)                        (q[i, j])         =  1 )
              % ( sum (j in rg)                        (q[j, i])         =  1 )
              1 #= sum(A[I, 1..N]),
              1 #= sum(A[1..N, I]),

              % ( sum (j, k in rg where j - k = i - 1) (q[j, k])         <= 1 )
          (
              multifor([J,K],1,N),
              fromto([], In, Out, List),
              param(A,I) do
          (
              LHS is J - K,
              RHS is I - 1,
              Elem is A[J,K],
              (LHS = RHS -> Out = [Elem | In] ; Out = In)
          )
          ),
          sum(List) #=< 1,

          % ( sum (j, k in rg where j - k = 1 - i) (q[j, k])         <= 1 )
          (
              multifor([J,K],1,N),
              fromto([], In, Out, List2),
              param(A,I) do
          (
              LHS is J - K,
              RHS is 1 - I,
              Elem is A[J,K],
              (LHS = RHS -> Out = [Elem | In] ; Out = In)
          )
          ),
          sum(List2) #=< 1,

          % ( sum (j, k in rg where j - k = i - 1) (q[n + 1 - j, k]) <= 1 )
          (
              multifor([J,K],1,N),
              fromto([], In, Out, List3),
              param(A,I,N) do
          (
              LHS is J - K,
              RHS is I - 1,
              Elem is A[N+1-J,K],
              (LHS = RHS -> Out = [Elem | In] ; Out = In)
          )
          ),
          sum(List3) #=< 1,

          % ( sum (j, k in rg where j - k = 1 - i) (q[n + 1 - j, k]) <= 1 )
          (
              multifor([J,K],1,N),
              fromto([], In, Out, List4),
              param(A,I,N) do
          (
              LHS is J - K,
              RHS is 1 - I,
              Elem is A[N+1-J,K],
              (LHS = RHS -> Out = [Elem | In] ; Out = In)
          )
          ),
          sum(List4) #=< 1
      ).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

problem_symmetries(Matrix, Syms) :-
        rows_reflect(Matrix, RowSym),
        columns_reflect(Matrix, ColSym),
        diagonal_reflect(Matrix, DiagSym),
        Syms = [RowSym, ColSym, DiagSym].

problem(N, Matrix) :-
        queens_bool(N, Matrix),
        problem_symmetries(Matrix, Syms),
        ldsb_initialise(Matrix, Syms).

none_problem(N, Matrix) :- queens_bool(N, Matrix).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gap_sbds_problem(N,M) :-
        queens_bool(N,M),
        ic_gap_sbds:sbds_initialise(M, [rows, cols], values:0..1,
                        [symmetry(d_4, [rows, cols], [])],
                        []).

gap_sbdd_problem(N,M) :-
        queens_bool(N,M),
        ic_gap_sbdd:sbdd_initialise(M, [rows, cols], values:0..1,
                        [symmetry(d_4, [rows, cols], [])],
                        []).

isbds_problem(N, M) :-
        queens_bool(N,M),
        problem_symmetries(M, Syms),
        convert_symmetries(pairs, M, Syms, SBDSSyms),
        ic_sbds:sbds_initialise(M, 2, SBDSSyms, #=, []).

static_problem(N, Matrix) :-
        queens_bool(N, Matrix),
        problem_symmetries(Matrix, Syms),
        post_symmetries(Matrix, Syms).

sbdd_ssb_problem(N, Vector, VarPart, ValPart) :-
        queens_bool(N, Matrix),
        problem_symmetries(Matrix, Syms),
        symmetries_to_ssb(Matrix, Syms, Vector, VarPart, ValPart).

